@@ -111,8 +111,6 @@ open(Dirname, Opts) ->
           %% and loading anyway: if later someone tries to write
           %% something, that someone will get a write_locked exception.
           _ = bitcask_lockops:delete_stale_lock(write, Dirname),
-          %% This purge will acquire the write lock prior to doing anything.
-          purge_setuid_files(Dirname),
           fresh;
         false -> undefined
     end,
@@ -124,7 +122,7 @@ open(Dirname, Opts) ->
     WaitTime = timer:seconds(get_opt(open_timeout, Opts)),
 
     %% Loop and wait for the keydir to come available.
-    case init_keydir(Dirname, WaitTime) of
+    case init_keydir(Dirname, WaitTime, WritingFile /= undefined) of
         {ok, KeyDir, ReadFiles} ->
             %% Ensure that expiry_secs is in Opts and not just application env
             ExpOpts = [{expiry_secs,get_opt(expiry_secs,Opts)}|Opts],
@@ -448,34 +446,39 @@ iterator_release(Ref) ->
 %%      into a more compact form.
 -spec merge(Dirname::string()) -> ok.
 merge(Dirname) ->
-    merge(Dirname, [], readable_files(Dirname)).
+    merge(Dirname, [], {readable_files(Dirname), []}).
 
 %% @doc Merge several data files within a bitcask datastore
 %%      into a more compact form.
 -spec merge(Dirname::string(), Opts::[_]) -> ok.
 merge(Dirname, Opts) ->
-    merge(Dirname, Opts, readable_files(Dirname)).
+    merge(Dirname, Opts, {readable_files(Dirname), []}).
 
 %% @doc Merge several data files within a bitcask datastore
 %%      into a more compact form.
 -spec merge(Dirname::string(), Opts::[_], FilesToMerge::[string()]) -> ok.
 merge(_Dirname, _Opts, []) ->
     ok;
-merge(Dirname, Opts, FilesToMerge0) ->
+merge(Dirname,Opts,FilesToMerge) when is_list(FilesToMerge) -> 
+    merge(Dirname,Opts,{FilesToMerge,[]});
+merge(_Dirname, _Opts, {[],_}) ->
+    ok;
+merge(Dirname, Opts, {FilesToMerge0, ExpiredFiles0}) ->
     %% Make sure bitcask app is started so we can pull defaults from env
     ok = start_app(),
-
     %% Filter the files to merge and ensure that they all exist. It's
     %% possible in some circumstances that we'll get an out-of-date
     %% list of files.
     FilesToMerge = [F || F <- FilesToMerge0,
                          filelib:is_file(F)],
-    merge1(Dirname, Opts, FilesToMerge).
+    ExpiredFiles = [F || F <- ExpiredFiles0,
+                         filelib:is_file(F)],
+    merge1(Dirname, Opts, FilesToMerge, ExpiredFiles).
 
 %% Inner merge function, assumes that bitcask is running and all files exist.
-merge1(_Dirname, _Opts, []) ->
+merge1(_Dirname, _Opts, [], _ExpiredFiles) ->
     ok;
-merge1(Dirname, Opts, FilesToMerge) ->
+merge1(Dirname, Opts, FilesToMerge, ExpiredFiles) ->
     %% Test to see if this is a complete or partial merge
     Partial = not(lists:usort(readable_files(Dirname)) == 
                   lists:usort(FilesToMerge)),
@@ -546,18 +549,22 @@ merge1(Dirname, Opts, FilesToMerge) ->
     TooNew = [F#file_status.filename ||
                  F <- Summary,
                  F#file_status.newest_tstamp >= MergeStart],
-    InFiles = lists:reverse(
-                lists:foldl(fun(F, Acc) ->
+    {InFiles,InExpiredFiles} = lists:foldl(fun(F, {InFilesAcc,InExpiredAcc} = Acc) ->
                                     case lists:member(F#filestate.filename,
                                                       TooNew) of
                                         false ->
-                                            [F|Acc];
+                                            case lists:member(F#filestate.filename,
+                                                    ExpiredFiles) of
+                                                false ->
+                                                    {[F|InFilesAcc],InExpiredAcc};
+                                                true ->
+                                                    {InFilesAcc,[F|InExpiredAcc]}
+                                            end;
                                         true ->
                                             bitcask_fileops:close(F),
                                             Acc
                                     end
-                            end, [], InFiles1)),
-
+                            end, {[],[]}, InFiles1),
     %% Setup our first output merge file and update the merge lock accordingly
     {ok, Outfile} = bitcask_fileops:create_file(Dirname, Opts),
     ok = bitcask_lockops:write_activefile(
@@ -581,6 +588,7 @@ merge1(Dirname, Opts, FilesToMerge) ->
                       opts = Opts },
 
     %% Finally, start the merge process
+    ExpiredFilesFinished = expiry_merge(InExpiredFiles, LiveKeyDir, []),
     State1 = merge_files(State),
 
     %% Make sure to close the final output file
@@ -589,11 +597,16 @@ merge1(Dirname, Opts, FilesToMerge) ->
 
     %% Close the original input files, schedule them for deletion,
     %% close keydirs, and release our lock
-    [bitcask_fileops:close(F) || F <- State#mstate.input_files],
+    [bitcask_fileops:close(F) || F <- State#mstate.input_files ++ ExpiredFilesFinished],
     {_, _, _, {IterGeneration, _, _}} = bitcask_nifs:keydir_info(LiveKeyDir),
-    FileNames = [F#filestate.filename || F <- State#mstate.input_files],
+    FileNames = [F#filestate.filename || F <- State#mstate.input_files ++ ExpiredFilesFinished],
     [catch set_setuid_bit(F) || F <- FileNames],
     bitcask_merge_delete:defer_delete(Dirname, IterGeneration, FileNames),
+    if InFiles == [] ->
+            bitcask_fileops:delete(Outfile);
+        true ->
+            ok
+    end,
 
     %% Explicitly release our keydirs instead of waiting for GC
     bitcask_nifs:keydir_release(LiveKeyDir),
@@ -606,8 +619,8 @@ consider_for_merge(FragTrigger, DeadBytesTrigger, ExpirationGraceTime) ->
     fun (F) ->
             (F#file_status.fragmented >= FragTrigger)
                 orelse (F#file_status.dead_bytes >= DeadBytesTrigger)
-                orelse (        (F#file_status.oldest_tstamp > 0)     %% means that the file has data
-                        andalso (F#file_status.oldest_tstamp < ExpirationGraceTime)
+                orelse ((F#file_status.oldest_tstamp > 0) andalso   %% means that the file has data
+                        (F#file_status.newest_tstamp < ExpirationGraceTime)
                        )
     end.
 
@@ -680,9 +693,9 @@ needs_merge(Ref) ->
                 _ ->
                     ok
             end,
-
             FileNames = [Filename || {Filename, _Reasons} <- MergableFiles],
-            {true, FileNames};
+            ExpiredFiles = [Filename || {Filename, [{data_expired,_,_}]} <- MergableFiles],
+            {true, {FileNames, ExpiredFiles}};
         false ->
             false
     end.
@@ -728,8 +741,8 @@ small_file_threshold(Opts) ->
 
 expired_threshold(Cutoff) ->
     fun(F) ->
-            if F#file_status.oldest_tstamp < Cutoff ->
-                    [{oldest_tstamp, F#file_status.oldest_tstamp, Cutoff}];
+            if F#file_status.newest_tstamp < Cutoff  ->
+                    [{data_expired, F#file_status.newest_tstamp, Cutoff}];
                true ->
                     []
             end
@@ -877,7 +890,7 @@ scan_key_files([Filename | Rest], KeyDir, Acc, CloseFile, EnoentOK) ->
 %%
 %% Initialize a keydir for a given directory.
 %%
-init_keydir(Dirname, WaitTime) ->
+init_keydir(Dirname, WaitTime, ReadWriteModeP) ->
     %% Get the named keydir for this directory. If we get it and it's already
     %% marked as ready, that indicates another caller has already loaded
     %% all the data from disk and we can short-circuit scanning all the files.
@@ -903,6 +916,13 @@ init_keydir(Dirname, WaitTime) ->
             Lock = poll_for_merge_lock(Dirname),
             try
                 poll_deferred_delete_queue_empty(),
+                if ReadWriteModeP ->
+                        %% This purge will acquire the write lock
+                        %% prior to doing anything.
+                        purge_setuid_files(Dirname);
+                   true ->
+                        ok
+                end,
                 init_keydir_scan_key_files(Dirname, KeyDir)
             after
                 ok = bitcask_lockops:release(Lock)
@@ -919,7 +939,7 @@ init_keydir(Dirname, WaitTime) ->
                 Value when is_integer(Value), Value =< 0 -> %% avoids 'infinity'!
                     {error, timeout};
                 _ ->
-                    init_keydir(Dirname, WaitTime - 100)
+                    init_keydir(Dirname, WaitTime - 100, ReadWriteModeP)
             end
     end.
 
@@ -994,7 +1014,7 @@ merge_files(#mstate {  dirname = Dirname,
                        [File#filestate.filename, Dirname, Error]),
                      State
              after
-                     catch bitcask_fileops:close(File)
+                    catch bitcask_fileops:close(File)
              end,
     merge_files(State2#mstate { input_files = Rest }).
 
@@ -1278,6 +1298,26 @@ poll_deferred_delete_queue_empty() ->
         _ -> receive after 1100 -> poll_deferred_delete_queue_empty() end
     end.
 
+%% Internal merge function for cache_merge functionality.
+expiry_merge([], _LiveKeyDir, Acc) ->
+    Acc;
+
+expiry_merge([File | Files], LiveKeyDir, Acc0) ->
+    FileId = bitcask_fileops:file_tstamp(File),
+    Fun = fun(K, Tstamp, {Offset, _TotalSz}, Acc) ->
+                bitcask_nifs:keydir_remove(LiveKeyDir, K, Tstamp, FileId, Offset),
+                Acc
+        end,
+    case bitcask_fileops:fold_keys(File, Fun, ok, default) of
+        {error, Reason} ->
+            error_logger:error_msg("Error folding keys for ~p: ~p\n", [File#filestate.filename,Reason]),
+            Acc = Acc0;
+        _ ->
+            error_logger:info_msg("All keys expired in: ~p scheduling file for deletion\n", [File#filestate.filename]),
+            Acc = lists:append(Acc0, [File])
+     end,
+    expiry_merge(Files, LiveKeyDir, Acc).
+
 %% ===================================================================
 %% EUnit tests
 %% ===================================================================
@@ -1678,9 +1718,9 @@ delete_partial_merge_test() ->
     %% selective merge, hit all of the files with deletes but not
     %%  all of the ones with deleted data
     timer:sleep(1100),
-    ok = merge("/tmp/bc.test.pardel",[],lists:reverse(lists:nthtail(2,
+    ok = merge("/tmp/bc.test.pardel",[],{lists:reverse(lists:nthtail(2,
                                            lists:reverse(readable_files(
-                                               "/tmp/bc.test.pardel"))))),
+                                               "/tmp/bc.test.pardel")))),[]}),
 
     %% Verify we've now only got one item left
     B2 = bitcask:open("/tmp/bc.test.pardel"),
